#include "design.h"

#include <vector>

#include "../ast_traverser/module_traverser.h"
#include "../ast_traverser/find_sc_main.h"
#include "../tools/misc.h"

#include "module_instance.h"

risc::Design::Design(int argc, char* argv[])
{
  project_ = frontend(argc, argv);

  if(project_ == NULL) {
    exit(1);
  }

  risc::GlobalDesign::design_static = this;

  sc_module_ = risc::tools::find_class_definition("sc_module","::sc_core::",project_);
  sc_port_   = risc::tools::find_class_definition("sc_port","::sc_core::",project_);
  sc_port_base_   = risc::tools::find_class_definition("sc_port_base","::sc_core::",project_);
  sc_in_     = risc::tools::find_class_definition("sc_in","::sc_core::",project_);
  sc_inout_  = risc::tools::find_class_definition("sc_inout","::sc_core::",project_);
  sc_out_    = risc::tools::find_class_definition("sc_out","::sc_core::",project_);
  sc_event_  = risc::tools::find_class_definition("sc_event","::sc_core::",project_);
  sc_prim_channel_ = risc::tools::find_class_definition("sc_prim_channel","::sc_core::",project_);
  sc_channel_ = risc::tools::find_class_definition("sc_channel","::sc_core::",project_);

  main_ = findMain(project_);
  sc_main_ = traverser::find_sc_main(project_);

  initialize_defintions();
  initialize_top_modules();
  initialize_declarations();
}

risc::Design::~Design()
{
  // clear all vectors

  for(ModuleVectorIter iter = module_definitions_.begin();
      iter != module_definitions_.end(); iter++) {
    delete *iter;
  }

  for(PrimitiveChannelVectorIter
      iter = primitive_channel_.begin();
      iter != primitive_channel_.end(); iter++) {
    delete *iter;
  }

  for(HierarchicalChannelVectorIter
      iter = hierarchical_channel_.begin();
      iter != hierarchical_channel_.end(); iter++) {
    delete *iter;
  }

  for(EventVectorIter iter = events_.begin(); iter != events_.end(); iter++ ) {
    delete *iter;
  }


  for(ThreadVectorIter iter = threads_.begin(); iter != threads_.end();
      iter++) {
    delete *iter;
  }

  for(CThreadVectorIter iter = cthreads_.begin(); iter != cthreads_.end();
      iter++) {
    delete *iter;
  }


  for(MethodVectorIter iter = methods_.begin(); iter != methods_.end();
      iter++) {
    delete *iter;
  }


  for(FunctionVectorIter  iter = functions_.begin(); iter != functions_.end();
      iter++) {
    delete *iter;
  }

}

void risc::Design::initialize_defintions()
{
  // Search for all class definitions that are derived from sc_module and
  // are not defined in a file in the include path
  std::vector<SgClassDefinition*> modules
    = risc::traverser::find_all_modules();

  for(std::vector<SgClassDefinition*>::iterator iter = modules.begin();
      iter != modules.end(); iter++) {

    SgClassDefinition* temp_cd = isSgClassDefinition(*iter);

    if(temp_cd) {

      module_definitions_.push_back(new Module(temp_cd));
    }
  }

  //FIXME Has to continue with channels, threads, ...
}

void risc::Design::initialize_declarations()
{
}

risc::Module* risc::Design::find_module_definition(SgClassDefinition* cd)
{
  Module* temp = NULL;

  for(ModuleVectorIter iter = module_definitions_.begin() ;
      iter != module_definitions_.end(); iter++) {

    if(risc::tools::class_types_are_identical((*iter)->get_ast_node(),cd)) {
      temp = *iter;
    }
  }

  return temp;
}

void risc::Design::initialize_top_modules()
{
  //FIXME get sc_main function and search for members that are variable
  //      instances and derived from sc_module

  SgFunctionDefinition* func_def = sc_main_->get_definition();
  SgBasicBlock* bb = func_def->get_body();

  for(Rose_STL_Container<SgStatement*>::iterator
      iter = bb->get_statements().begin();
      iter != bb->get_statements().end();
      iter++) {

    SgVariableDeclaration* var_decl =  isSgVariableDeclaration(*iter);

    if(var_decl) {

      SgClassDefinition* class_def =
        tools::class_definition_of_variable(var_decl);

      if(tools::is_derived_from_sc_module(class_def)) {

        Module* md = find_module_definition(class_def);

        top_modules_.push_back(new ModuleInstance(var_decl->get_definition(), md));
      }
    }
  }
}

void risc::Design::print_design()
{
  for(risc::ModuleInstanceVectorIter
      iter = top_modules_.begin();
      iter != top_modules_.end(); iter++) {

    std::cout <<  (*iter)->get_ast_type_name()
              << " "
              << (*iter)->get_name()
              << std::endl;

    (*iter)->definition_->print_hierarchy(1);
  }
}

void risc::Design::print_all_class_definitions()
{
  for(ModuleVectorConstIter iter = module_definitions_.begin();
      iter != module_definitions_.end(); iter++) {

    std::cout << (*iter)->get_ast_type_name() << std::endl;
  }
}

void risc::Design::print_all_primitive_channel_definitions()
{
  for(PrimitiveChannelVectorConstIter
      iter = primitive_channel_.begin();
      iter != primitive_channel_.end(); iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

void risc::Design::print_all_hierarchical_channel_definitions()
{
  for(HierarchicalChannelVectorConstIter
      iter = hierarchical_channel_.begin();
      iter != hierarchical_channel_.end(); iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

void risc::Design::print_all_events()
{
  for(EventVectorConstIter iter = events_.begin(); iter != events_.end();
      iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

void risc::Design::print_all_threads()
{
  for(ThreadVectorConstIter iter = threads_.begin();
    iter != threads_.end(); iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

void risc::Design::print_all_cthreads()
{
  for(CThreadVectorConstIter iter = cthreads_.begin();
    iter != cthreads_.end(); iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

void risc::Design::print_all_methods()
{
  for(MethodVectorConstIter iter = methods_.begin();
    iter != methods_.end(); iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

void risc::Design::print_all_functions()
{
  for(FunctionVectorConstIter iter = functions_.begin();
    iter != functions_.end(); iter++) {
//    std::cout << get_name(*iter) << std::endl;
  }
}

risc::Design* risc::GlobalDesign::design_static = NULL;

risc::Design* risc::get_design()
{
  return risc::GlobalDesign::design_static;
}

SgProject* risc::get_project()
{
  return risc::GlobalDesign::design_static->project_;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
