#ifndef MODULE_H_INCLUDED_
#define MODULE_H_INCLUDED_

#include<iostream>
#include<string>
#include<vector>

#include "module_instance.h"
#include "variable.h"

#include "function.h"
#include "thread.h"
#include "cthread.h"
#include "method.h"

#include "hierarchical_channel.h"
#include "hierarchical_channel_instance.h"
#include "primitive_channel.h"
#include "primitive_channel_instance.h"

#include "event.h"

#include "inport.h"
#include "inoutport.h"
#include "outport.h"

#include "class.h"


namespace risc {

class Module;
typedef std::vector<Module*> ModuleVector;
typedef std::vector<Module*>::iterator ModuleVectorIter;
typedef std::vector<Module*>::const_iterator ModuleVectorConstIter;

/**
 *  \class Module
 *  \brief This class represents a definition of a module
 *
 *  A module is defined by the assembling of modules, channels, ports,
 *  processes, and others.
 */
class Module: public Class {

public:

 /**
  * Default constructor
  *
  * \brief Creates an empty module definition. All the lists of instances
  *        and sub definitions are empty.
  */
  explicit Module(SgClassDefinition *ast_node);

  ~Module();


  ///////////////////////// initialization funcitons ///////////////////////// 

  /**
   * \brief  This function searches for all variable declarations in the module
   *         definition. The corresponding variable vectors will be used.
   */
  void initialize_variables();

  /**
   * \brief  This function searches for all functions, SC_THREADS, SC_CTHREADS,
   *         and SC_METHODS  in the module definition. The corresponding
   *         variable vectors will be used.
   */
  void initialize_functions();

  /////////////////////////////  helper functions //////////////////////////// 

  /**
   * \brief  Prints the complete module design into the terminal
   */
  void print_module_definition();

  /**
   * \brief  Prints the hierarchy. Roots is this definiton
   */
  void print_hierarchy(unsigned indent_level = 0);

  /**
   * \brief  Find an event by it's ast variable definition
   */
  Event* find_event(SgVariableDefinition* var_def);


  ////////////////////  Vectors module definition elements /////////////////// 

  /**
   * \brief Vector of C++ builtin variables that are declared in the module
   */
  VariableVector variables_;

  /**
   * \brief Vector of module definitions which are defined in this module.
   */
  ModuleVector sub_modules_defs_;

  /**
   * \brief Vector of module definitions which are defined in this module.
   */
  ModuleInstanceVector sub_modules_;

  /**
   * \brief Vector of primimitive channel instances which are declared in this
   *        module
   */
  PrimitiveChannelInstanceVector sub_prim_channel_;

  /**
   * \brief Vector of hierarchical channel instances which are declared in this
   *        module
   */
  HierarchicalChannelInstanceVector sub_hier_channel_;

  /**
   * \brief Vector of primitive channel definitions. Technically, this
   *        represents a nested C++ class definition. In a good design this
   *        vector should be empty.
   */
  PrimitiveChannelVector sub_prim_channel_defs_;

  /**
   * \brief Vector of hierarchical channel definitions. Technically, this
   *        represents a nested C++ class definition. In a good design this
   *        vector should be empty.
   */
  HierarchicalChannelVector sub_hier_channel_defs_;

  /**
   * \brief Vector of threads which are defined in this module
   */
  ThreadVector threads_;

  /**
   * \brief Vector of cthreads which are defined in this module
   */
  CThreadVector cthreads_;

  /**
   * \brief Vector of methods which are defined in this module
   */
  MethodVector methods_;

  /**
   * \brief Vector of functions which are defined in this module
   */
  FunctionVector functions_;

  /**
   * \brief Vector of inports which are defined in this module
   */
  InPortVector inports_;

  /**
   * \brief Vector of inoutports which are defined in this module
   */
  InOutPortVector inoutports_;

  /**
   * \brief Vector of outports which are defined in this module
   */
  OutPortVector outports_;

  /**
   * \brief Vector of events which are defined in this module
   */
  EventVector events_;

private:

  /**
   * \brief Disabled copy constructor
   */
  Module(const Module &md);
};

} // end of namespace risc

#endif /* MODULE_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
